import 'package:flutter/material.dart' show Color, TextStyle, Rect, Canvas, Size, CustomPainter, Paint, Gradient, LinearGradient, Alignment, Offset, FilterQuality, Colors, TextSpan, TextPainter, TextDirection;
import 'package:hunterchart/lib/style/chart_style.dart';
import 'model/hunter_chart_model.dart';

double? _lastPriceNow = 0;
int _statusNow = 0;
double _max = 0;
double _min = double.infinity;
double? currentsY, oldsY;
double? currentsX, oldsX;

abstract class BaseChart extends CustomPainter {
  /// linear Gradient
  List<Color> bgColor;
  int rows, columns;
  HunterChartStyle? chartStyle;
  double? spaceText;
  double padding;

  Paint gridPaint = Paint()
    ..isAntiAlias = true
    ..filterQuality = FilterQuality.high
    ..strokeWidth = 0.5
    ..color = Colors.white;

  BaseChart({required this.bgColor, required this.columns, required this.rows, this.chartStyle, this.spaceText, required this.padding});

  @override
  void paint(Canvas canvas, Size size) {
    initialStyle();
    initChart();
    background(canvas, size);
    textRight(canvas, size);
    chart(canvas, size);
  }

  void initialStyle() {
    gridPaint = Paint()
      ..isAntiAlias = true
      ..filterQuality = FilterQuality.high
      ..strokeWidth = chartStyle!.gridStrokeWidth ?? 0.5
      ..color = chartStyle!.gridColor ?? Colors.white;
  }

  void background(Canvas canvas, Size size) {
    Paint paintBg = Paint();
    Gradient colorBg = LinearGradient(
      begin: Alignment.bottomCenter,
      end: Alignment.topCenter,
      colors: bgColor,
    );
    Rect sizeRectangle = Rect.fromLTRB(0, 0, size.width, size.height);
    canvas.save();
    canvas.drawRect(sizeRectangle, paintBg..shader = colorBg.createShader(sizeRectangle));
    canvas.restore();
  }

  void initChart();
  void textRight(Canvas canvas, Size size);
  void chart(Canvas canvas, Size size);

  void showPriceNow(Offset offset, Canvas canvas, Size size, {String? priceNow}) {
    TextSpan span = TextSpan(text: "${priceNow ?? '0'}", style: TextStyle(color: Colors.white, fontSize: 11));
    TextPainter tp = TextPainter(text: span, textDirection: TextDirection.rtl);
    if (_lastPriceNow! < double.parse(priceNow!)) {
      _statusNow = 0;
    } else if (_lastPriceNow! > double.parse(priceNow)) {
      _statusNow = 1;
    }
    Paint gridPriceNowPaint = Paint()
      ..isAntiAlias = true
      ..filterQuality = FilterQuality.high
      ..strokeWidth = 1
      ..color = _statusNow == 0 ? Colors.green : Colors.red;
    if (_lastPriceNow != double.parse(priceNow)) _lastPriceNow = double.parse(priceNow);
    canvas.drawLine(offset, Offset(0, offset.dy), gridPriceNowPaint);
    canvas.drawRect(Rect.fromLTRB(size.width - spaceText! - (padding + 4), offset.dy - 10, size.width, offset.dy + 10), gridPriceNowPaint);
    tp.layout();
    tp.paint(canvas, Offset(size.width - spaceText!, offset.dy - (tp.height - (tp.height / 2))));
  }

  searchingMax(List<HunterChartData> data) {
    int maxLength = 0;
    for (var i in data) {
      if (_max < i.close!) _max = i.close!;
      if (_min > i.close!) _min = i.close!;
      if (_min == 0) _min = double.infinity;
      if (maxLength < i.close!.toString().length) maxLength = i.close!.toString().length;
    }
    if (_max == _min) _min -= _max - ((_max / rows));
    if (maxLength < _min.toString().length) maxLength = _min.toString().length;
    return {'max': _max, 'min': _min, 'length': maxLength};
  }

  findY(List x, double value) {
    var maxHeight = 0.0, minHeight = 0.0;
    var a = 0.0, b = 0.0;
    var newRangeY = 0.0, oldRangeY = 0.0;
    double resultY = 0;
    x.reversed.forEach((element) {
      element.forEach((k, v) {
        if (_max >= k && maxHeight < 1) maxHeight = v;
        if (_min <= k && minHeight < 1) minHeight = v;
        if (a < k) {
          if (b != a) {
            b = a;
            oldRangeY = newRangeY;
          }
          a = k;
          newRangeY = v;
        }

        if (a >= value && b <= value) {
          resultY = oldRangeY - v;
          // diff
          if (a <= value) {
            resultY = v;
          } else if (b >= value)
            resultY = oldRangeY;
          else
            resultY = v + (resultY * (1 - (value / a)));
          return;
        } else if (value >= _max)
          resultY = maxHeight;
        else if (value <= _min) resultY = minHeight;
      });
      if (resultY > 0) {
        return;
      }
    });
    return resultY;
  }

  @override
  bool shouldRepaint(BaseChart oldDelegate) {
    return true;
  }
}
