import 'package:hunterchart/lib/animation.dart';
import 'package:hunterchart/lib/style/chart_style.dart';

import 'base_chart.dart';
import 'package:flutter/material.dart' show Canvas, Rect, Color, Size, TextSpan, TextStyle, Colors, TextPainter, TextDirection, Paint, Offset, Gradient, LinearGradient, Alignment, Path, Shader, TileMode, PaintingStyle;

import 'model/hunter_chart_model.dart';

getYOffset() => [currentsY, oldsY];
getXOffset() => [currentsX, oldsX];

class BuildChart extends BaseChart {
  HunterChartStyle? chartStyle;
  List<HunterChartData>? data;
  List<Color> bgColor;
  int rows, columns;
  var min = 1.0, max = 100.0;
  int lengthMax;
  List savedY = [];
  double? dragOffset;
  AnimateChartNow? animationCircle;
  AnimateChartText? animationChartText;
  AnimateChartLine? animationLineX;
  AnimateChartLine? animationLineY;

  /// auto decision
  double? spaceText = 30;
  double padding = 12;

  BuildChart({required this.bgColor, this.rows = 4, this.columns = 4, this.lengthMax = 5, this.data, required this.chartStyle, this.spaceText, this.dragOffset, this.padding = 12, this.animationCircle, this.animationChartText, this.animationLineX, this.animationLineY}) : super(bgColor: bgColor, columns: columns, rows: rows, chartStyle: chartStyle, spaceText: spaceText, padding: padding);

  @override
  void initChart() {
    // if (animationCircle == null) {
    //   animationCircle = AnimateChartNow();
    // }
    if (chartStyle == null) chartStyle = HunterChartStyle();
    spaceText = 30;
    var resultMaxMin = searchingMax(data!);
    if (resultMaxMin != null) {
      min = resultMaxMin['min'];
      max = resultMaxMin['max'];
      lengthMax = resultMaxMin['length'];
    }
  }

  @override
  void textRight(Canvas canvas, Size size) {
    double rangeNumberText = 0;
    if (max != min) rangeNumberText = (max - min) / (rows - 1);
    double rowSpace = size.height / (rows);
    for (var i = 0; i <= rows; i++) {
      double spaceRight = 35;
      double textRange;
      if (i == rows)
        textRange = min;
      else {
        if (i == 0)
          textRange = min + (rangeNumberText * (rows - i));
        else
          textRange = min + (rangeNumberText * (rows - i));
      }
      // mencari jarak kanan
      if (textRange.toStringAsFixed(3).length > 5) spaceRight += (6.5 * (textRange.toStringAsFixed(3).length - 5));
      if (spaceText! < spaceRight) spaceText = spaceRight;
      TextSpan span = TextSpan(text: "${textRange.toStringAsFixed(3)}", style: TextStyle(color: Colors.white, fontSize: 11));
      TextPainter tp = TextPainter(text: span, textDirection: TextDirection.rtl);
      // canvas.drawLine(Offset(0, 0), Offset(0, 0), gridPaint);
      tp.layout();
      if (i == rows) {
        tp.paint(canvas, Offset(size.width - spaceRight, ((rowSpace * (i - 1))) - tp.height));
        // tp.paint(canvas, Offset(si1ze.width - spaceRight, size.height - tp.height));
        savedY.add({textRange: ((rowSpace * (i - 1))) - tp.height});
      } else {
        if (i == 0) {
          // escape
        } else if (min < textRange && i < rows - 1) {
          tp.paint(canvas, Offset(size.width - spaceRight, ((rowSpace * i)) - (tp.height - (tp.height / 2))));
          savedY.add({textRange: ((rowSpace * i)) - (tp.height - (tp.height / 2))});
        }
      }
    }
  }

  void drawLine(canvas, size) {}

  @override
  void chart(canvas, size) {
    // draw area
    Paint paintBg = Paint();
    Gradient colorBg = LinearGradient(
      begin: Alignment.bottomCenter,
      end: Alignment.topCenter,
      colors: [Colors.black45, Colors.black45],
    );
    Rect sizeRectangle = Rect.fromLTRB(0, 0, (size.width - spaceText!) - padding, size.height);
    canvas.save();
    canvas.drawRect(sizeRectangle, paintBg..shader = colorBg.createShader(sizeRectangle));
    // draw grid
    canvas.clipRect(Rect.fromLTRB(0, 0, size.width - spaceText! - padding, size.height));
    canvas.translate(-1 * (dragOffset! > 0 ? dragOffset! : 0), 0);
    double rowSpace = size.height / rows;
    for (int i = 0; i <= rows; i++) {
      canvas.drawLine(Offset(0, rowSpace * i), Offset((size.width * 2) - spaceText! - padding, rowSpace * i), gridPaint);
    }

    double columnSpace = ((size.width) - spaceText!) / columns;
    for (int i = 0; i < columns; i++) {
      canvas.drawLine(Offset(columnSpace * i, 0), Offset(columnSpace * i, size.height), gridPaint);
    }

    Paint mLineFillPaint = Paint()
      ..style = PaintingStyle.fill
      ..isAntiAlias = true;
    Shader mLineFillShader = LinearGradient(
      begin: Alignment.topCenter,
      end: Alignment.bottomCenter,
      tileMode: TileMode.clamp,
      colors: [chartStyle!.fillChartColor ?? Colors.white, Colors.transparent],
    ).createShader(sizeRectangle);
    mLineFillPaint..shader = mLineFillShader;
    Path mLineFillPath = Path();
    double currentY = 0, oldY = 0, oldSpace = 0;
    int? currentCounter;
    Paint mLinePaint = Paint()
      ..isAntiAlias = true
      ..style = PaintingStyle.stroke
      ..strokeWidth = 1
      ..color = chartStyle!.lineChartColor ?? Colors.white;

    Path path = Path();
    for (var i = 0; i < data!.length; i++) {
      final y = findY(savedY, data![i].close!);
      if (currentCounter != i) {
        currentCounter = i;
        if (oldY != currentY) {
          oldY = currentY;
          oldSpace = columnSpace * (i - 1);
        }
        currentY = y;
      }
      // path.moveTo(columnSpace * (i + 1), oldY);
      // path.cubicTo(((columnSpace * (i - 1)) + (columnSpace * i)) / 2, currentY, ((columnSpace * (i - 1)) + (columnSpace * i)) / 2, currentY, columnSpace * i, currentY);
      if (i == data!.length - 2) {
        oldsY = y;
        oldsX = columnSpace * i;
      }
      if (i == data!.length - 1) {
        currentsY = y;
        currentsX = columnSpace * i;
        path.moveTo(oldSpace, oldY);
        // curve
        if (chartStyle!.lineStyle == LineStyle.CURVE || chartStyle!.lineStyle == null) {
          path.cubicTo((oldSpace + animationLineX!.newValue!) / 2, oldY, (oldSpace + animationLineX!.newValue!) / 2, animationLineY!.newValue!, columnSpace * i, animationLineY!.newValue!);
        } else if (chartStyle!.lineStyle == LineStyle.SHARP) {
          path.cubicTo(columnSpace * i, animationLineY!.newValue!, columnSpace * i, animationLineY!.newValue!, columnSpace * i, animationLineY!.newValue!);
        }
      } else if (i != 0) {
        path.moveTo(oldSpace, oldY);
        // curve
        if (chartStyle!.lineStyle == LineStyle.CURVE || chartStyle!.lineStyle == null) {
          path.cubicTo((oldSpace + (columnSpace * i)) / 2, oldY, (oldSpace + (columnSpace * i)) / 2, y, columnSpace * i, y);
        } else if (chartStyle!.lineStyle == LineStyle.SHARP) {
          path.cubicTo(columnSpace * i, y, columnSpace * i, y, columnSpace * i, y);
        }
      }

      // canvas.drawLine(Offsedft(columnSpace * i, 0), Offset(columnSpace * i, y), gridPaint);
      if (i == data!.length - 1) {
        if (chartStyle!.fillChart == null ? true : chartStyle!.fillChart!) {
          mLineFillPath.moveTo(oldSpace, size.height);
          mLineFillPath.lineTo(oldSpace, oldY);
          if (chartStyle!.lineStyle == LineStyle.CURVE || chartStyle!.lineStyle == null) {
            mLineFillPath.cubicTo((oldSpace + animationLineX!.newValue!) / 2, oldY, (oldSpace + animationLineX!.newValue!) * 0.5, animationLineY!.newValue!, columnSpace * i, animationLineY!.newValue!);
          } else if (chartStyle!.lineStyle == LineStyle.SHARP) {
            mLineFillPath.cubicTo((animationLineX!.newValue!), animationLineY!.newValue!, (animationLineX!.newValue!), animationLineY!.newValue!, columnSpace * i, animationLineY!.newValue!);
          }
          mLineFillPath.lineTo(columnSpace * i, size.height);
          mLineFillPath.close();
        }
      } else {
        if (chartStyle!.fillChart == null ? true : chartStyle!.fillChart!) {
          mLineFillPath.moveTo(oldSpace, size.height);
          mLineFillPath.lineTo(oldSpace, oldY);
          if (chartStyle!.lineStyle == LineStyle.CURVE || chartStyle!.lineStyle == null) {
            mLineFillPath.cubicTo((oldSpace + (columnSpace * i)) / 2, oldY, (oldSpace + (columnSpace * i)) * 0.5, y, columnSpace * i, y);
          } else if (chartStyle!.lineStyle == LineStyle.SHARP) {
            mLineFillPath.cubicTo(((columnSpace * i)), y, ((columnSpace * i)), y, columnSpace * i, y);
          }
          mLineFillPath.lineTo(columnSpace * i, size.height);
          mLineFillPath.close();
        }
      }

      // TextSpan span = TextSpan(text: "", style: TextStyle(color: Colors.white, fontSize: 11));
      // TextPainter tp = TextPainter(text: span, textDirection: TextDirection.rtl);

      // return;
      if (i + 1 == data!.length) {
        canvas.drawCircle(Offset(animationLineX!.newValue!, animationLineY!.newValue!), 3, Paint()..color = Colors.white);
        canvas.drawCircle(Offset(animationLineX!.newValue!, animationLineY!.newValue!), animationCircle!.radius ?? 3, Paint()..color = Color.fromRGBO(255, 255, 255, animationCircle!.opacity ?? 1));
      }
    }
    if (chartStyle!.fillChart == null ? true : chartStyle!.fillChart!) {
      canvas.drawPath(mLineFillPath, mLineFillPaint);
    }
    canvas.drawPath(path, mLinePaint..strokeWidth);
    path.reset();
    // mLineFillPath.moveTo(lastX, chartRect.height + chartRect.top);
    // mLineFillPath.lineTo(lastX, getY(lastPrice));
    // mLineFillPath.cubicTo((lastX + curX) / 2, getY(lastPrice), (lastX + curX) / 2, getY(curPrice), curX, getY(curPrice));
    // mLineFillPath.lineTo(curX, chartRect.height + chartRect.top);

    mLineFillPath.reset();
    canvas.restore();
    showPriceNow(Offset(size.width, animationLineY!.newValue!), canvas, size, priceNow: animationChartText!.rangeValue.toString());
  }
}
