import 'dart:async';

import 'package:hunterchart/lib/animation.dart';
import 'package:hunterchart/lib/model/hunter_chart_model.dart';
import 'package:hunterchart/lib/style/chart_style.dart';

import 'build_chart.dart';
import 'package:flutter/material.dart';

double _close = 0;

class HunterChart extends StatefulWidget {
  final List<Color>? bgColor;
  final int rows, columns;
  final List<HunterChartData>? data;
  final HunterChartStyle? chartStyle;
  final bool? withCircleAnimation, textRangeAnimation;

  HunterChart({Key? key, this.bgColor, this.rows = 4, this.columns = 4, this.data, this.chartStyle, this.withCircleAnimation, this.textRangeAnimation}) : super(key: key);

  @override
  _HunterChartState createState() => _HunterChartState();
}

class _HunterChartState extends State<HunterChart> with TickerProviderStateMixin {
  late BuildChart chart;
  List queue = [false, false];
  List lastqueue = [0, 0];
  bool onDrag = false;
  double dragOffset = 0, startDragOffset = 0, currentDragOffset = 0;
  Animation<double>? animationCircle;
  AnimationController? controllerCircle;
  Animation<double>? animationOpacity;
  AnimationController? controllerOpacity;
  Animation<double>? animationTextRange;
  AnimationController? controllerTextRange;
  Tween<double>? textRangeValue;
  AnimationController? controllerYOffset;
  Animation<double>? animateYOffset;
  Tween<double>? rangeY;
  AnimationController? controllerXOffset;
  Animation<double>? animateXOffset;
  Tween<double>? rangeX;

  _getNewValue() {
    final _last = widget.data!.last.close;
    if (_close != _last) {
      textRangeValue = Tween(begin: _close, end: _last);
      _close = _last!;
      controllerTextRange = AnimationController(vsync: this, duration: Duration(milliseconds: 1000));
      animationTextRange = textRangeValue!.animate(CurvedAnimation(parent: controllerTextRange!, curve: Curves.easeInOut)
        ..addListener(() {
          setState(() {});
        })
        ..addStatusListener((status) {
          if (status == AnimationStatus.completed) {
            controllerTextRange!.stop();
          }
        }));
      if (widget.textRangeAnimation ?? true) {
        controllerTextRange!.forward();
      }
    }

    final offsetY = getYOffset();
    final lastY = offsetY[0] ?? 0.0;
    final oldY = offsetY[1] ?? 0.0;
    final offsetX = getXOffset();
    final lastX = offsetX[0] ?? 0.0;
    final oldX = offsetX[1] ?? 0.0;
    if (lastqueue[0] != lastY) {
      if (lastY != oldY) {
        // queue[0] = true;
        rangeY = Tween(begin: oldY, end: lastY);
        controllerYOffset = AnimationController(vsync: this, duration: Duration(milliseconds: 700));
        animateYOffset = rangeY!.animate(CurvedAnimation(parent: controllerYOffset!, curve: Curves.easeInOut)
          ..addListener(() {
            setState(() {});
          })
          ..addStatusListener((status) {
            if (status == AnimationStatus.completed) {
              lastqueue[0] = lastY;
            }
          }));
        if (widget.textRangeAnimation ?? true) {
          controllerYOffset!.forward();
        }
      }
    }

    if (lastqueue[1] != lastX) {
      if (lastX != oldX) {
        rangeX = Tween(begin: oldX, end: lastX);

        controllerXOffset = AnimationController(vsync: this, duration: Duration(milliseconds: 700));
        animateXOffset = rangeX!.animate(CurvedAnimation(parent: controllerXOffset!, curve: Curves.easeInOut)
          ..addListener(() {
            setState(() {});
          })
          ..addStatusListener((status) {
            lastqueue[1] = lastX;
          }));
        if (widget.textRangeAnimation ?? true) {
          controllerXOffset!.forward();
        }
      }
    }
  }

  @override
  void initState() {
    super.initState();
    // animasi opacity
    controllerOpacity = AnimationController(vsync: this, duration: Duration(milliseconds: 2000));
    animationOpacity = Tween<double>(begin: 0.6, end: 0).animate(CurvedAnimation(parent: controllerOpacity!, curve: Curves.easeInOut)
      ..addListener(() {
        setState(() {});
      })
      ..addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          Timer(Duration(milliseconds: 2000), () {
            controllerOpacity!.reset();
            controllerOpacity!.forward();
          });
        }
      }));
    // animasi scale
    controllerCircle = AnimationController(vsync: this, duration: Duration(milliseconds: 2000));
    animationCircle = Tween<double>(begin: 3, end: 60).animate(CurvedAnimation(curve: Curves.easeInOut, parent: controllerCircle!)
      ..addListener(() {
        setState(() {});
      })
      ..addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          Timer(Duration(milliseconds: 2000), () {
            controllerCircle!.reset();
            controllerCircle!.forward();
          });
        } else if (status == AnimationStatus.dismissed) {
          controllerCircle!.forward();
        }
      }));
    if (widget.withCircleAnimation ?? true) {
      controllerOpacity!.forward();
      controllerCircle!.forward();
    }
  }

  @override
  void dispose() {
    controllerOpacity!.dispose();
    controllerCircle!.dispose();
    controllerTextRange!.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    _getNewValue();
    chart = BuildChart(
      bgColor: widget.bgColor ?? [Color(0xff000000), Color(0xff000000)],
      rows: widget.rows,
      columns: widget.columns,
      data: widget.data!,
      chartStyle: widget.chartStyle,
      dragOffset: currentDragOffset,
      animationCircle: AnimateChartNow(
        opacity: animationOpacity!.value,
        radius: animationCircle!.value,
      ),
      animationChartText: AnimateChartText(
        rangeValue: animationTextRange!.value,
      ),
      animationLineY: animateYOffset == null ? null : AnimateChartLine(newValue: animateYOffset!.value),
      animationLineX: animateXOffset == null ? null : AnimateChartLine(newValue: animateXOffset!.value),
    );

    return GestureDetector(
      child: CustomPaint(
        size: Size(double.infinity, double.infinity),
        painter: chart,
      ),
      onHorizontalDragStart: (drag) {
        if (!onDrag) onDrag = true;
        startDragOffset = drag.globalPosition.dx;
      },
      onHorizontalDragUpdate: (drag) {
        dragOffset = startDragOffset - drag.globalPosition.dx;
        currentDragOffset -= dragOffset * -0.01;
        if (currentDragOffset < 0) currentDragOffset = 0;
        setState(() {});
      },
      onHorizontalDragEnd: (drag) {
        if (onDrag) onDrag = false;
        startDragOffset = 0;
        dragOffset = 0;
      },
    );
  }
}
